#include "tim_helper.hpp"
#include <stdio.hpp>

namespace JabyEngine {
    namespace FileProcessor {
        namespace {
            using GPU::AreaU16;

            struct BlockInfo {
                uint32_t size;
                uint16_t x;
                uint16_t y;
                uint16_t w;
                uint16_t h;
            };

            struct TIMState : public TIMFileProcessor::GenericTIM {                
                // TODO: Create function???
                TIMState() {
                }

                virtual Progress parse_header(State::Configuration& config) override {
                    static constexpr auto HEADER_SIZE = 2*sizeof(uint32_t);

                    if(config.data_bytes >= (HEADER_SIZE + sizeof(BlockInfo))) {
                        BlockInfo block_info;

                        config.processed(HEADER_SIZE);
                        Helper::simple_read(block_info, config);
                        this->clut_area = AreaU16::create(block_info.x, block_info.y, block_info.w, block_info.h);
                        return Progress::Done;
                    }
                    return Progress::Error;
                }

                virtual Progress pre_data_parsing(State::Configuration& config) {
                    if(config.data_bytes >= sizeof(BlockInfo)) {
                        BlockInfo block_info;

                        Helper::simple_read(block_info, config);
                        this->tex_area = AreaU16::create(block_info.x, block_info.y, block_info.w, block_info.h);
                        return Progress::Done;
                    }
                    return Progress::InProgress;
                }
            };
        }

        State create(const uint32_t* data_adr, const TIM& file) {
            using Callback = Progress(*)(State::Configuration& config, TIMState& simple_tim);
            return State::from(TIMState(), data_adr, reinterpret_cast<Callback>(TIMFileProcessor::parse_header));
        }
    }
}